#!/usr/bin/perl -w

=head1 NAME

extractpdb.pl - Script to decode a "Bundesliga" PDB file into its textual
representation

=head1 SYNOPSIS

extractpdb.pl <pdb file>

=head1 DESCRIPTION

Use this script to create a textual representation of a Bundesliga PDB file
suitable to be used with genpdb.pl. The script accepts PDB files for
Bundesliga 3.0 and newer.

=head1 COPYRIGHT

extractpdb.pl (c) Thomas Pundt 2002

Permission to use, copy, modify, and distribute this software for any
purpose, without fee, and without a written agreement is hereby granted,
provided that the above copyright notice and this paragraph and the
following two paragraphs appear in all copies.

IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

=cut

use PDA::Pilot; # from the pilot-link package.
use POSIX qw(mktime);

$secs_since_70 = 24107 * 86400 + 3600;
%penalty = ( 1=>"aO",2=>"aP" );

sub usage()
{
  print "Usage: $0 <pdb file>\n";
  exit;
}

usage() if ($#ARGV == -1);

$file = PDA::Pilot::File::open($ARGV[0]) or usage();

$dbinfo = $file->getDBInfo();
print "name:$dbinfo->{name}\n";

$record = $file->getRecord(0);
($Version,$matchDay,$NumTeams,$NumMatchDays,$options,$favTeam,
 $pwin,$pdraw,$ploss,$ppenalty,
 $tb[0],$tb[1],$tb[2],$tb[3],$tb[4],$tb[5],$tb[6],$tb[7],
 $col[0]->{from},$col[0]->{to},$col[0]->{color},
 $col[1]->{from},$col[1]->{to},$col[1]->{color},
 $col[2]->{from},$col[2]->{to},$col[2]->{color},
 $col[3]->{from},$col[3]->{to},$col[3]->{color},
 $col[4]->{from},$col[4]->{to},$col[4]->{color},
) = unpack('nnnnccccccCCCCCCCCCCCCCCCCCCCCCCC',$record->{raw});

die "can't handle DB files prior to version 6 - this has version $Version.\n" if ($Version<6);

if (0) {
  print "Matchday:$matchDay\n".
        "NumTeams:$NumTeams\n".
        "Options:$options\n";
}

if ($Version == 8) {
  print "pointsWin:$pwin\n".
        "pointsDraw:$pdraw\n".
        "pointsLoss:$ploss\n".
        "pointsPenalty:$ppenalty\n".
        "tiebreaker:".join("|",@tb)."\n".
        "ranges:";
  for $obj (@col) {
    print "[".$obj->{from}."|".$obj->{to}."|".$obj->{color}."]";
  }
  print "\n";
}

print "\n";
$record = $file->getRecord(1);
@teams = split(/\0/,$record->{raw});
$teams[$favTeam-1] .= ":fav" if ($favTeam > 0);
print join("\n",@teams) . "\n";

for (2..$NumMatchDays+1) {
  $record = $file->getRecord($_);
  ($numResults, $numHandicaps) = unpack('cc',$record->{raw});
  print "\n";
  for $rec (0..$numResults-1) {
    if ($Version == 8) {
      ($t1,$t2,$r1,$r2,$date,$penalty) = unpack('CCCCNn',substr($record->{raw},2+$rec*10));
    }
    if ($Version < 8) {
      ($t1,$t2,$r1,$r2,$date) = unpack('CCCCN',substr($record->{raw},2+$rec*8));
    }
    if ($date > $secs_since_70) {
      $date -= $secs_since_70;
      my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($date);
      $datestring = sprintf "%04d%02d%02d%02d%02d", 
                            $year+1900, $mon+1, $mday, $hour-$isdst, $min;
    } else {
      $datestring = "000000000000";
    }
    $resstring = "";
    if ($r1 != 255 && $r2 != 255) {
      $resstring = "$r1:$r2";
      if ($Version == 8 and $penalty > 0) {
	$resstring .= ":$penalty{$penalty}";
      }
    }
    printf "$datestring:$teams[$t1-1]:$teams[$t2-1]:$resstring\n";
  }
  for $hrec (0..$numHandicaps-1) {
    if ($Version == 8) {
      ($type,$team,$val) = unpack('ccc',substr($record->{raw},$numResults*10+$hrec*3+2));
    }
    if ($Version < 8) {
      ($type,$team,$val) = unpack('ccc',substr($record->{raw},$numResults*8+$hrec*3+2));
    }
    printf ":%s:%s:%d\n", $type==0?"=ph":"=gh",$teams[$team-1],$val;
  }
}
