/*
 * Bundesliga is Copyright (c) 1997-2003 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "bl.h"
#include "blrcp.h"

#define FROMMATCHDAY 0
#define FROMTABLE    1

Int16 matchDay, DBNo;
UInt32 Date;
TeamResultType *TeamResult = 0, *BackupTeamResult = 0;
MemHandle TeamResultHandle, BackupTeamResultHandle;
BlDatabasePreferencesType prefs;
BlApplicationPreferencesType appPrefs;
Boolean deviceCanBeam;

/*
 * array that holds pointers to the fields in the main form. t1 and t2 are
 * used to show the encounter, r1 and r2 hold the result. team1 and team2
 * store the teams's index.
 */
struct {
  FieldType *t1, *t2, *r1, *r2;
  Int8 team1, team2;
  UInt32 Date;
  UInt16 penalty;
} match[12];

static TablePtr leagueTable;
static TablePtr gotoTable;
static TablePtr fixtureTable;

statsType stats;

static Int16 TableRememberPosFirst = 0;
static Boolean showDay = true;

static Boolean ComputeTable(UInt32) TBLFUNC;
static UInt32 computePartition(void) TBLFUNC;
static Int16 cmpByPosition(void*, void*, Int32) TBLFUNC;
static char cmpTeamsByTiebreaker(TeamResultType *a, TeamResultType *b, UInt8) TBLFUNC;
static Boolean isHomeMatch(matchType *m, Int8 team, Int8 matchDay) TBLFUNC;
static Boolean isAwayMatch(matchType *m, Int8 team, Int8 matchDay) TBLFUNC;
static Boolean inBetween(matchType *m, Int8 team, Int8 matchDay) TBLFUNC;
static void InitFormBl(void) TBLFUNC;
static void SetDateLabel(Int8 n) TBLFUNC;
static void toggleMatchdayLabel(Boolean toggle) TBLFUNC;
static void SetResultHandle(FieldType* fptr, UInt8 n1, UInt8 n2) TBLFUNC;
static UInt32 GetMostUsedDate(void) TBLFUNC;
static void SaveMatchdayResults(void) TBLFUNC;
static void AddTeamResult(Int8 n, Int8 r1, Int8 r2, UInt16 penalty, Boolean home) TBLFUNC;
static Boolean EvalResult(matchType *m, Int8 matchday) TBLFUNC;
static Boolean EvalHandicap(handicapType *h, Int8 i) TBLFUNC;
static void DrawTableItem(void*, Int16, Int16, RectangleType*) TBLFUNC;
static void InitFormTable(void) TBLFUNC;
static void InitTable(void) TBLFUNC;
static void InitFormGoto(void) TBLFUNC;
static void DrawFixtureItem(void*, Int16, Int16, RectangleType*) TBLFUNC;
static Int16 InitFormShowFixture(void) TBLFUNC;
static void DrawIntAt(Int16 v, Coord x, Coord y) TBLFUNC;
static void GetGamesString(Char* buf, Int16 n) TBLFUNC;

/*
 * Redraw table pointed to by "tblP";
 * Arguments:
 * - tblP:     pointer to the table
 * - first:    number of the first row to be shown
 * - rows:     total number of rows in the table
 * - up, down: ID's of "up" and "down" elements in the form
 *             to move the table
 */
void RedrawTable(TablePtr tblP, UInt16 first,
		 Int8 rows, UInt16 up, UInt16 down)
{
  UInt16 row, last = first+9;
  ControlType *upP = Id2Ptr(up);
  ControlType *downP = Id2Ptr(down);
  char *label;

  if (last>rows-1) last=rows-1;
  for (row=0; row<TblGetNumberOfRows(tblP); row++)
    TblSetRowUsable(tblP, row, 
		    (row>=first && row<=last) ? true : false);

  label = (char*)CtlGetLabel(upP);
  label[0] = (first==0) ? 3 : 1;
  label = (char*)CtlGetLabel(downP);
  label[0] = (last==rows-1) ? 4 : 2;
  
  CtlDrawControl(upP);
  CtlDrawControl(downP);

  TblMarkTableInvalid(tblP);
  TblRedrawTable(tblP);
}

static Int16 cmpByPosition(void *a, void *b, Int32 other)
{
  if (((TeamResultType*)a)->Position > ((TeamResultType*)b)->Position) return 1;
  if (((TeamResultType*)a)->Position < ((TeamResultType*)b)->Position) return -1;
  return 0;
}

static char cmpTeamsByTiebreaker(TeamResultType *a, TeamResultType *b, UInt8 t)
{
  switch (t) {
  case tieBreakerPoints:
    if (a->Points > b->Points) return 1;
    if (a->Points < b->Points) return -1;
    break;
  case tieBreakerGoalsDiff:
    if (a->Goals + b->AGoals > b->Goals + a->AGoals) return 1;
    if (a->Goals + b->AGoals < b->Goals + a->AGoals) return -1;
    break;
  case tieBreakerGoals:
    if (a->Goals > b->Goals) return 1;
    if (a->Goals < b->Goals) return -1;
    break;
  case tieBreakerMatchesWon:
    if (a->Won > b->Won) return 1;
    if (a->Won < b->Won) return -1;
    break;
  }
  return 0;
}

static Boolean isHomeMatch(matchType *m, Int8 team, Int8 matchDay)
{
  return team == m->t1;
}

static Boolean isAwayMatch(matchType *m, Int8 team, Int8 matchDay)
{
  return team == m->t2;
}

static Boolean inBetween(matchType *m, Int8 team, Int8 matchDay)
{
  return (matchDay>=stats.first && matchDay<=stats.last);
}

/**
 * fixture form functions
 */
static void InitFormBl(void)
{
  Int16 i, fixup = 0, fixupX = 0, offset = 0, rows;

  for (i=0; i<12; i++) {
    match[i].t1 = Id2Ptr(ID_FrmTeam+i);
    match[i].t2 = Id2Ptr(ID_FrmOpp+i);
    match[i].r1 = Id2Ptr(ID_FrmRes+i);
    match[i].r2 = Id2Ptr(ID_FrmOppRes+i);
    FldSetUsable(match[i].t1, false);
    FldSetUsable(match[i].t2, false);
    FldSetUsable(match[i].r1, false);
    FldSetUsable(match[i].r2, false);
    CtlSetUsable(Id2Ptr(ID_FrmBlDateButton+i), false);
  }

  rows = prefs.NumTeams>>1;
  if (rows == 11) fixup = 1;
  if (rows == 10) fixup = 2;
  if (rows  < 10) { 
    fixup = 2;
    offset = 5;
  }
  if (prefs.pointsPenalty == 0) {
    fixupX = 1;
    for (i=0; i<12; i++) {
      FrmResizeObject(ID_FrmTeam+i,62,10+fixup);
      CtlSetUsable(Id2Ptr(ID_FrmBlPenaltyTrigger+i), false);
    }
  }

  if (fixup || fixupX) {
    for (i=0; i<12; i++) {
      FrmMoveObject(ID_FrmTeam+i,0,i*fixup+offset);
      FrmMoveObject(ID_FrmOpp+i,fixupX*5,i*fixup+offset);
      FrmMoveObject(ID_FrmRes+i,fixupX*11,i*fixup+offset);
      FrmMoveObject(ID_FrmOppRes+i,fixupX*13,i*fixup+offset);
      FrmMoveObject(ID_FrmBlDateButton+i,fixupX*5,i*fixup+offset);
      FrmMoveObject(ID_FrmBlPenaltyTrigger+i,0,i*fixup+offset);
      FrmMoveObject(ID_FrmBlPenaltyTrigger+20+i,0,i*fixup+offset);
    }
    for (i=0; i<5; i++) {
      FrmMoveObject(ID_FrmButton+i,0,-fixup);
    }
  }

  ChooseDB();

  stats.predicate = inBetween;
  stats.first = 0;
  stats.last = prefs.NumMatchDays - 1;
}

/*
 * set "date/matchday switching button" with date of match[n]
 */
static void SetDateLabel(Int8 n)
{
  DateTimeType dateP;
  char label[17];

  showDay = false;

  if (match[n].Date > 0) {
    TimSecondsToDateTime(match[n].Date, &dateP);
    DateToAscii(dateP.month,dateP.day,dateP.year,
	        PrefGetPreference(prefDateFormat),
		label);

    // don't draw 0:00 time
    if (dateP.hour+dateP.minute>0) {
      StrCat(label," ");
      TimeToAscii(dateP.hour,dateP.minute,
		  PrefGetPreference(prefTimeFormat),
		  label+StrLen(label));
    }
  } else {
    StrCopy(label, "");
  }
  ButtonSetLabel(ID_FrmLabelMatchday,label);
}

/*
 * toggle between match day and most used date of match day
 */
static void toggleMatchdayLabel(Boolean toggle)
{
  char label[17];
  char md[3];

  if (toggle)
    showDay = !showDay;

  if (showDay || !Date) {
    char *p = GetString(ID_tSTR_Matchday);
    StrIToA(md, matchDay);
    StrCopy(label,p);
    StrCat(label, " ");
    StrCat(label, md);
  } else {
    DateTimeType dateP;
    TimSecondsToDateTime(Date, &dateP);
    DateToAscii(dateP.month,dateP.day,dateP.year,
		PrefGetPreference(prefDateFormat), label);
  }
  ButtonSetLabel(ID_FrmLabelMatchday,label);
}

static void SetResultHandle(FieldType* fptr, UInt8 n1, UInt8 n2)
{
  MemHandle h;
  if (!(h = FldGetTextHandle(fptr))) {
    h = MemHandleNew(4);
  }
  if (n1==255 && n2==255)
    StrCopy(MemHandleLock(h),"");
  else
    StrIToA(MemHandleLock(h), n1);
  MemHandleUnlock(h);
  FldSetTextHandle(fptr, h);
}

/*
 * Get the most used date from one match day.
 * hmm; way too long for such a simple thing :-/
 */
static UInt32 GetMostUsedDate(void)
{
  struct count {
    UInt32 date;
    Int8 num;
  } date[12];

  UInt32 Date;
  Int16 num = 0, i, j;
  Boolean newDate;

  for (i=0; i<12; i++) {
    if (match[i].Date == 0)
      continue;
    newDate = true;
    for (j=0; j<num; j++) {
      if (date[j].date == match[i].Date) {
	date[j].num++;
	newDate = false;
	break;
      }
    }
    if (newDate) {
      date[num].date = match[i].Date;
      date[num].num = 1;
      num++;
    }
  }

  Date = 0;
  i = 0;
  for (j=0; j<num; j++) {
    if (date[j].num > i) {
      Date = date[j].date;
      i = date[j].num;
    }
  }
  return Date;
}

void DrawMatches(Boolean redraw)
{
  Int16 i;
  Int8 numTeams, numHandicaps;
  char *p, *q;
  matchType *m;
  SelectedDBInfo info;
  FormPtr frm = FrmGetActiveForm();
  ListPtr pLst;

  p = q = MemHandleLock(DmQueryRecord(db,matchDay+1));

  numTeams = (Int8)*p++;
  numHandicaps = (Int8)*p++;

  for (i=0; i<12; i++) {

    match[i].team1 = match[i].team2 = match[i].Date = 0;

    if (i<numTeams) {
      m = (matchType*)p;
      match[i].Date = m->Date;
      match[i].penalty = m->penalty;
    
      if ( m->t1==0 || m->t2==0 || 
	   m->t1>prefs.NumTeams || m->t2>prefs.NumTeams ) {
	DisplayErrorMessageAndStop(ID_ErrorMessages);
	MemPtrUnlock(q);
	prefs.Version = 0; // don't use this file again...
	return;
      }

      FldSetTextPtr(match[i].t1, theTeams[m->t1-1]);
      FldSetTextPtr(match[i].t2, theTeams[m->t2-1]);
      
      SetResultHandle(match[i].r1,m->r1,m->r2);
      SetResultHandle(match[i].r2,m->r2,m->r1);
      
      match[i].team1 = m->t1;
      match[i].team2 = m->t2;

      if (prefs.pointsPenalty) {
	pLst = Id2Ptr(ID_FrmBlPenaltyTrigger+20+i);
	LstSetSelection(pLst, m->penalty);
	ButtonSetLabel(ID_FrmBlPenaltyTrigger+i,
		       LstGetSelectionText(pLst,LstGetSelection(pLst)));
	CtlSetUsable(Id2Ptr(ID_FrmBlPenaltyTrigger+i), true);
      }

      p += sizeof(matchType);

      FldSetUsable(match[i].t1, true);
      FldSetUsable(match[i].t2, true);
      FldSetUsable(match[i].r1, true);
      FldSetUsable(match[i].r2, true);
      CtlSetUsable(Id2Ptr(ID_FrmBlDateButton+i), true);
    } else {
      
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmTeam+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmOpp+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmRes+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmOppRes+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmBlDateButton+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmBlPenaltyTrigger+i));
    }
  }

  Date = GetMostUsedDate();

  MemPtrUnlock(q);

  SelectedDB(DBNo, &info);
  FrmCopyLabel(frm, ID_FrmPopup, info.dbName);
  if (redraw) {
    FrmDrawForm(frm);
  }
  toggleMatchdayLabel(false);
}

/*
 * get the contents of the editable fields in the form 
 * and save them to database.
 */
static void SaveMatchdayResults(void)
{
  Int16 i;
  char *r1, *r2;
  char *p, *q;
  Int8 numTeams;
  Boolean dirty = false;
  matchType m;
  Int8 e1, e2;

  if (!db || prefs.options & readonly)
    return;

  FrmSetFocus(FrmGetActiveForm(), -1);

  p = q = MemHandleLock(DmGetRecord(db,matchDay+1));
  numTeams = (Int8)*p;
  p += 2;

  for (i=0; i<numTeams; i++) {
    if (i>=12)
      break;

    MemMove(&m,p,sizeof(matchType));

    r1 = FldGetTextPtr(match[i].r1);
    r2 = FldGetTextPtr(match[i].r2);

    e1 = e2 = -1;

    if (r1 && StrLen(r1) && r2 && StrLen(r2)) {
      e1 = StrAToI(r1);
      e2 = StrAToI(r2);
    }

    if (e1 != m.r1 || e2 != m.r2 || match[i].penalty != m.penalty) {
      m.r1 = e1;
      m.r2 = e2;
      m.penalty = match[i].penalty;
      DmWrite(q,2+i*sizeof(matchType),&m,sizeof(matchType));
      dirty = true;
    }
    p += sizeof(matchType);
  }
  MemPtrUnlock(q);
  DmReleaseRecord(db,matchDay+1,dirty);
}

Boolean MatchdayFormHandleEvent(EventPtr e)
{
  static Int16 first, last;
  Boolean handled;
  SelectedDBInfo info;
  Int16 t1, t2, item;
  FieldType *r1p, *r2p;
  
  handled = false;
  
  switch (e->eType) {

  case frmOpenEvent:
    InitFormBl();
    first = 0;
    last = prefs.NumMatchDays - 1;
    DrawMatches(true);
    handled = true; 
    break;

  case menuEvent:
    SaveMatchdayResults();
    switch(e->data.menu.itemID) {

    // About Bundesliga
    case ID_MenuItem:
      FrmPopupForm(ID_FrmAbout);
      handled = true; 
      break;

    // Edit team names, set fav team
    case ID_MenuItem + 1:
      FrmPopupForm(ID_FrmTeams);
      handled = true; 
      break;

    // Edit Fixtures
    case ID_MenuItem + 2:
      FrmPopupForm(ID_FrmEdit);
      handled = true; 
      break;

    // Delete Database
    case ID_MenuItem + 3:
      if (FrmAlert(ID_FrmDBDelete) == 0) { /* Delete */
        TableRememberPosFirst = 0;
	if ( DeleteDatabase(DBNo) 
	     == (appErrorClass|noDatabaseInstalled) ) {
	  DisplayErrorMessageAndStop(ID_ErrorMessages+5);
	} else {
	  DBNo = 0;
	  SelectedDB(DBNo, &info);
	  OpenDatabase(0, 0);
	  matchDay = prefs.matchDay;
	  DrawMatches(false);
	  FrmGotoForm(ID_FrmBl);
	}
      }
      handled = true; 
      break;

    // Create new database
    case ID_MenuItem + 4:
      FrmPopupForm(ID_FrmNewDB);
      handled = true; 
      break;

    // Edit league names, order of leagues
    case ID_MenuItem + 5:
      FrmPopupForm(ID_FrmLeague);
      handled = true; 
      break;

    // Settings form
    case ID_MenuItem + 6:
      FrmPopupForm(ID_FrmSettings);
      handled = true; 
      break;

    // Edit handicaps
    case ID_MenuItem + 7:
      FrmPopupForm(ID_FrmHandicap);
      handled = true; 
      break;

    // Beam database
    case ID_MenuItem + 8:
      BeamDatabase(DBNo);
      handled = true; 
      break;

    // DB Settings form
    case ID_MenuItem + 9:
      FrmGotoForm(ID_FrmDatabaseSettings);
      handled = true; 
      break;

    }
    break;

  case ctlSelectEvent:
    switch (e->data.ctlEnter.controlID) {

    // Matchday/Date label upper right corner
    case ID_FrmLabelMatchday:  
      toggleMatchdayLabel(true);
      handled = true; 
      break;

    // Table button
    case ID_FrmButton:
      FrmGotoForm(ID_FrmTable);
      stats.first = first = 0;
      stats.last  = last = prefs.NumMatchDays - 1;
      stats.evalHandicap = true;
      handled = true; 
      break;

    // different table views (home/away/...) button
    case ID_FrmButton+4:
      {
	ListPtr pLst = Id2Ptr(ID_FrmPopup+2);
	LstSetSelection(pLst, 0);
	LstPopupList(pLst);
	switch (LstGetSelection(pLst)) {
	case 0: // table for day "matchDay"
	  first = 0;
	  last = matchDay-1;
	  handled = true; 
	  break;
	case 1: // second round matches only
	  first = prefs.NumMatchDays>>1;
	  last = prefs.NumMatchDays - 1;
	  handled = true; 
	  break;
	case 2: // last 5 match days
	  first = matchDay - 5;
	  if (first<0) first = 0;
	  last = matchDay-1;
	  handled = true; 
	  break;
	case 3: // home matches
	  first = 0;
	  last = prefs.NumMatchDays - 1;
	  handled = true; 
	  stats.predicate = isHomeMatch;
	  break;
	case 4: // away matches
	  first = 0;
	  last = prefs.NumMatchDays - 1;
	  handled = true; 
	  stats.predicate = isAwayMatch;
	  break;
	}
	if (handled) {
	  FrmGotoForm(ID_FrmTable);
	  stats.first = first;
	  stats.last  = last;
	  stats.evalHandicap = false;
	}
      }
      break;

    case ID_FrmButton+1: /* prev match day button */
      if (matchDay>1) {
	SaveMatchdayResults();
        --matchDay;
        DrawMatches(true);
      }
      handled = true; 
      break;
    
    case ID_FrmButton+2: /* next match day button */
      if (matchDay < prefs.NumMatchDays) {
        SaveMatchdayResults();
        ++matchDay;
        DrawMatches(true);
      }
      handled = true; 
      break;

    case ID_FrmButton+3: /* jump to button */
      FrmGotoForm(ID_FrmGoto);
      handled = true; 
      break;

    // select a different database/league file
    case ID_FrmPopup:
      {
	Int16 item;
	ListPtr pLst = Id2Ptr(ID_FrmPopup+1);
	LstSetSelection(pLst, DBNo);
	LstMakeItemVisible(pLst, DBNo);
	LstPopupList(pLst);
	if ( (item=LstGetSelection(pLst)) != DBNo && item!=-1) {
          TableRememberPosFirst = 0;
	  DBNo = item;
	  SaveMatchdayResults();
	  SelectedDB(DBNo, &info);
	  SaveStatus(DBNo);
	  if ( OpenDatabase(info.cardNo, info.dbID) ) {
	    DBNo = 0;
	    SelectedDB(DBNo, &info);
	    OpenDatabase(0,0);
	  }
	  matchDay = prefs.matchDay;
	  DrawMatches(false);
	  FrmGotoForm(ID_FrmBl);
	}
	handled = true;
      }

    default:
      // one of the "-" buttons between the team names
      if (e->data.ctlEnter.controlID >= ID_FrmBlDateButton &&
	  e->data.ctlEnter.controlID <= ID_FrmBlDateButton+11) {
	SetDateLabel(e->data.ctlEnter.controlID - ID_FrmBlDateButton);
      }

      if (e->data.ctlEnter.controlID >= ID_FrmBlPenaltyTrigger &&
	  e->data.ctlEnter.controlID <= ID_FrmBlPenaltyTrigger+11) {
	ListPtr pLst = Id2Ptr(e->data.ctlEnter.controlID+20);
	LstPopupList(pLst);
        ButtonSetLabel(e->data.ctlEnter.controlID,
		       LstGetSelectionText(pLst,LstGetSelection(pLst)));
	match[e->data.ctlEnter.controlID-ID_FrmBlPenaltyTrigger].penalty 
	  = LstGetSelection(pLst);
        handled = true;
      }
    }
    break;

  case keyDownEvent:
    switch (e->data.keyDown.chr) {
    case pageUpChr:
      if (matchDay>1) {
        SaveMatchdayResults();
        --matchDay;
        DrawMatches(true);
      }
      handled = true;
      break;

    case pageDownChr:
      if (matchDay < prefs.NumMatchDays) {
        SaveMatchdayResults();
        ++matchDay;
        DrawMatches(true);
      }
      handled = true; 
    }
    break;

  // tapped on a team name
  case fldEnterEvent:
    item = e->data.fldEnter.fieldID-ID_FrmTeam;
    t1 = 0;
    r1p = r2p = 0;
    if (item>=0 && item<12) {
      t1 = match[item].team1;
      t2 = match[item].team2;
      r1p = match[item].r1;
      r2p = match[item].r2;
    } else if ( (item += ID_FrmTeam-ID_FrmOpp)>=0 
	       && item<12) {
      t1 = match[item].team2;
      t2 = match[item].team1;
      r1p = match[item].r2;
      r2p = match[item].r1;
    }
    if (t1>0) {
      handled = true;
      // if editMode, increase result of the team
      if ( (appPrefs.options & editMode) && !(prefs.options & readonly) ) {
	Int16 r1 = 0, r2 = 0, n1, n2;
	// suggested by Rolf Muth <rolf.muth@ruhr-uni-bochum.de>
	if ( r1p && FldGetTextLength(r1p)) 
	  r1 = ( StrAToI(FldGetTextPtr(r1p)) + 1 ) % 255;
	if ( r2p )
	  r2 = StrAToI(FldGetTextPtr(r2p)) % 255;
	n1 = r1 + (r2<<8);
	n2 = r2 + (r1<<8);
	SetResultHandle(r1p,n1,n2);
	SetResultHandle(r2p,n2,n1);
	//FrmDrawForm(FrmGetActiveForm());
	FldDrawField(r1p);
	FldDrawField(r2p);
      } else { // if not editMode, show statistics
	stats.team = t1;
	stats.first = first;
        stats.last = last;
	stats.evalHandicap = true;
	if (ComputeTable(0xffffffff))
	  FrmPopupForm(ID_FrmStatistics);
      }
    }
    break;

  case frmCloseEvent:
    SaveMatchdayResults();
    break;

  default:
    break;
  }

  return handled;
}

/**
 * general Compute Table functions
 */

static void AddTeamResult(Int8 n, Int8 r1, Int8 r2, UInt16 penalty, Boolean home)
{
  Int8 pointsWon  = prefs.pointsWon;  
  Int8 pointsDraw = prefs.pointsDraw;  
  Int8 pointsLost = prefs.pointsLost;
  if (prefs.pointsPenalty && penalty) {
    pointsWon  = prefs.pointsDraw+prefs.pointsPenalty;
    pointsLost = prefs.pointsDraw;
  }

  if (n>0) n--;

  TeamResult[n].Goals  += r1;
  TeamResult[n].AGoals += r2;
  TeamResult[n].Matches++;
  if (r1>r2) {
    TeamResult[n].Points+=pointsWon;
    TeamResult[n].Won++;
  } else if (r1==r2) {
    TeamResult[n].Points+=pointsDraw;
    TeamResult[n].Rem++;
  } else {
    TeamResult[n].Points+=pointsLost;
    TeamResult[n].Lost++;
  }

  if (home) {
    TeamResult[n].HGoals  += r1;
    TeamResult[n].HAGoals += r2;
    TeamResult[n].HMatches++;
    if (r1>r2) {
      TeamResult[n].HPoints+=pointsWon;
      TeamResult[n].HWon++;
    } else if (r1==r2) {
      TeamResult[n].HPoints+=pointsDraw;
      TeamResult[n].HRem++;
    } else {
      TeamResult[n].HPoints+=pointsLost;
      TeamResult[n].HLost++;
    }
  }
}

static Boolean EvalResult(matchType *m, Int8 matchday)
{
  if ( m->t1==0 || m->t2==0 || 
       m->t1>prefs.NumTeams || m->t2>prefs.NumTeams ) {
    DisplayErrorMessageAndStop(ID_ErrorMessages);
    return false;
  }
  
  if (m->r1 != 0xff && m->r2 != 0xff) {
    if (stats.predicate(m,m->t1,matchday)) {
      AddTeamResult(m->t1,m->r1,m->r2,m->penalty,true);
    }
    if (stats.predicate(m,m->t2,matchday)) {
      AddTeamResult(m->t2,m->r2,m->r1,m->penalty,false);
    }
  }
  return true;
}

static Boolean EvalHandicap(handicapType *h, Int8 i)
{
  if (h->type == pointHandicap) {
    TeamResult[h->team-1].Points += h->val;
  } else {
    TeamResult[h->team-1].Goals += h->val;
  }
  return true;
}

/*
 * Loop over all match days specified in "stats",
 * apply match and handicap callback functions for 
 * every match and handicap.
 * So this function adds up all goals and points of
 * the league.
 */
Boolean LoopRecords(matchTypeCallback mCallback, 
		    handicapTypeCallback hCallback, UInt32 mask)
{
  Int16 i, j;
  Int8 numHandicaps, numTeams;
  char *p, *q;
  matchType *m;
  handicapType *h;

  for (i=0; i<=stats.last; i++) {

    p = q = MemHandleLock(DmQueryRecord(db,i+2));
    numTeams = (Int8)*p++;
    numHandicaps = (Int8)*p++;
    
    if (mCallback) {
      for (j=0; j<numTeams; j++) {
	m = (matchType*)p;

	// first check mask, that team is involved
	if ( (mask & (1UL<<(m->t1-1))) && 
	     (mask & (1UL<<(m->t2-1))) ) {
	
	  if (!mCallback(m, i)) {
	    MemPtrUnlock(q);
	    prefs.Version = 0; // don't use this file again...
	    return false;
	  }

	}
	
	p += sizeof(matchType);
      }
    } else {
      p += numTeams*sizeof(matchType);
    }

    if (stats.evalHandicap && hCallback) {
      for (j=0; j<numHandicaps; j++) {
	h = (handicapType*)p;
	// check if team is involved
	if ( (mask & (1UL<<(h->team-1))) )
	  hCallback(h, i);
	p += sizeof(handicapType);
      }
    }

    MemPtrUnlock(q);
  }

  return true;
}

/*
 * compute the positions where in the table the ranking changes
 */
static UInt32 computePartition(void)
{
  Int8 prevPosition = -1;
  UInt32 partition = 0, i;

  SysInsertionSort(TeamResult,
		   prefs.NumTeams,
		   sizeof(TeamResultType),
		   cmpByPosition,
		   0);
  for (i=0; i<prefs.NumTeams; i++) {
    if (TeamResult[i].Position != prevPosition) {
      partition = partition | (1UL<<i);
      prevPosition = TeamResult[i].Position;
    }
  }
  return partition;
}

/*
 * compute the table of a league according to the "stats"
 * settings.
 */
static Boolean ComputeTable(UInt32 teamMask)
{
  Int16 i,j,t;
  UInt32 partition;
  Int16 partitionStartPoint, partitionEndPoint;
  Boolean res;

  if (teamMask == 0xffffffff)
    TeamResult = MemHandleLock(TeamResultHandle);

  // first step: reset previous values to zero.
  for (i=0; i<prefs.NumTeams; i++) {
    TeamResult[i].Goals = TeamResult[i].AGoals = TeamResult[i].Points 
      = TeamResult[i].Matches = TeamResult[i].HMatches = TeamResult[i].Won
      = TeamResult[i].HWon = TeamResult[i].Rem = TeamResult[i].HRem = TeamResult[i].Lost
      = TeamResult[i].HLost = TeamResult[i].HGoals = TeamResult[i].HAGoals 
      = TeamResult[i].HPoints = 0;
    TeamResult[i].teamNo = i+1;
    TeamResult[i].Position = 0;
  }

  // second step: loop over all records necessary and sum up
  // goals, points and handicaps.
  if ( (res = LoopRecords(EvalResult,EvalHandicap,teamMask)) ) {

    partition = 1; // initial partition of table

    // we do table/partition sorting for each tiebreaker
    for (t=0; t<8; t++) {
      if (prefs.tiebreaker[t] != tieBreakerNone) {

	// compute start and end position of a partition;
	// initial partition is the whole table.
	partitionStartPoint = 0;
	for ( partitionEndPoint = partitionStartPoint+1; 
	      partitionEndPoint <= prefs.NumTeams; 
	      partitionEndPoint++ ) {
	  if ( (partition & (1UL<<partitionEndPoint)) || 
	       partitionEndPoint == prefs.NumTeams) {

	    // now sort the partition by selected tiebreaker.

	    // handle "direct encounter" differently, because a separate
	    // "sub table" has to be built with all teams involved.
	    if (prefs.tiebreaker[t] == tieBreakerDirectCompare 
		&& partitionEndPoint-partitionStartPoint > 1) { // direct comparison 
	      
	      UInt32 mask = 0;

	      // if we have recursively come to this point, there is
	      // not much to check. Procede with next tiebreaker.
	      if (teamMask != 0xffffffff) 
		continue;

	      // build mask of involved teams
	      for ( i=partitionStartPoint; i<partitionEndPoint; i++) {
		mask = mask | (1UL<< (TeamResult[i].teamNo-1));
	      }

	      // save TeamResult table for later reference
	      BackupTeamResult = MemHandleLock(BackupTeamResultHandle);
	      MemMove(BackupTeamResult, TeamResult, prefs.NumTeams*sizeof(TeamResultType));

	      // compute sub table, evaluate results
	      ComputeTable(mask);
	      for ( i=partitionStartPoint; i<partitionEndPoint; i++) {
		for (j=0; j<prefs.NumTeams; j++) {
		  if (BackupTeamResult[i].teamNo == TeamResult[j].teamNo)
		    BackupTeamResult[i].Position += TeamResult[j].Position;
		}
	      }

	      // restore original "corrected" TeamResult.
	      MemMove(TeamResult, BackupTeamResult, prefs.NumTeams*sizeof(TeamResultType));
	      MemHandleUnlock(BackupTeamResultHandle);

	    } else {
	      char sort;
	      for ( i=partitionStartPoint; i<partitionEndPoint-1; i++) {
		if ( (teamMask & (1UL<<(TeamResult[i].teamNo-1))) ) {
		  for (j=i+1; j<partitionEndPoint; j++) {
		    if ( (teamMask & (1UL<<(TeamResult[j].teamNo-1))) ) { 
		      sort = cmpTeamsByTiebreaker(&TeamResult[i],
						  &TeamResult[j],
						  prefs.tiebreaker[t]);
		      if (sort<0) TeamResult[i].Position++;
		      if (sort>0) TeamResult[j].Position++;
		    }
		  }
		}
	      }
	    }

	    partitionStartPoint = partitionEndPoint;
	  }
	}

	partition = computePartition();
      }
    }

  }
  if (teamMask == 0xffffffff)
    MemHandleUnlock(TeamResultHandle);

  return res;
}

/**
 * Form Compute Table Functions
 */

static void DrawTableItem(void* tableP, Int16 row, Int16 column, 
                            RectangleType *bounds)
{
  Int16 n, i;
  UInt16 x;
  Char buf[25]; // "buf" holds the string to be drawn
  IndexedColorType color = 255, take;

  n = TblGetItemInt(tableP, row, column);
  x = bounds->topLeft.x;

  StrIToA(buf,n);

  // use background colors for devices that can handle them
  // to draw special table positions.
  if (FeatureSet35) {
    for (i=0; i<5; i++) {
      if ( (prefs.colors[i].color != 255 || appPrefs.color[i] != 255)
	   && prefs.colors[i].from <= row+1 
	   && prefs.colors[i].to >= row+1 ) {
	take = prefs.colors[i].color;
	if (take == 255) 
	  take = appPrefs.color[i];
	color = WinSetBackColor(take);
	bounds->extent.x++;
	WinEraseRectangle(bounds,0);
      }
    } 
  }

  switch (column) {

  // Position of team
  case 0:
    StrCat(buf,".");
    break;

  // Draw team name, and if fav team, also underline it
  case 1:
    TeamResult = MemHandleLock(TeamResultHandle);
    StrCopy(buf, theTeams[TeamResult[n].teamNo-1]);
    if (TeamResult[n].teamNo == prefs.favTeam)
      WinSetUnderlineMode(solidUnderline);
    MemHandleUnlock(TeamResultHandle);
    break;

  // case2: "buf" already holds num of matches played.
  // case4: "buf" already holds num of goals against team.
  case 3:
    StrCat(buf," :");
    x += bounds->extent.x - FntCharsWidth(buf,StrLen(buf)) - 1;
    break;
  }

  FntSetFont(0);
  WinDrawChars(buf, StrLen(buf), x, bounds->topLeft.y);
  WinSetUnderlineMode(noUnderline);
  if (color != 255 && FeatureSet35) {
      WinSetBackColor(color);
  }

  // use lines to draw special table positions on devices
  // that can't handle background colors.
  if (!FeatureSet35) {
    for (i=0; i<5; i++) {
      if ( prefs.colors[i].to == row+1 ) {
	WinDrawLine(bounds->topLeft.x,
		    bounds->topLeft.y+bounds->extent.y-1,
		    bounds->topLeft.x+bounds->extent.x,
		    bounds->topLeft.y+bounds->extent.y-1);
      }
      if ( prefs.colors[i].from == row+1 ) {
	WinDrawLine(bounds->topLeft.x,
		    bounds->topLeft.y-1,
		    bounds->topLeft.x+bounds->extent.x,
		    bounds->topLeft.y-1);
      }
    }
  }
}  

/*
 * initialize table variables to default values;
 */
static void InitFormTable(void)
{
  Int16 i, j;
  leagueTable = Id2Ptr(ID_FrmTableTable);

  for (i=0; i<prefs.NumTeams; i++) {
    TblSetRowSelectable(leagueTable, i, true);
    for (j=0; j<6; j++) {
      TblSetItemStyle(leagueTable, i, j, customTableItem);
      TblSetColumnUsable(leagueTable, j, true);
      TblSetCustomDrawProcedure(leagueTable, j, DrawTableItem);
    }
  }
  for (i=prefs.NumTeams; i<TblGetNumberOfRows(leagueTable); i++) {
    TblSetRowUsable(leagueTable, i, false);
  }
}

/*
 * fills in the values for the table and redraws it.
 */
static void InitTable(void)
{
  Int16 n;
 
  TeamResult = MemHandleLock(TeamResultHandle);
  for (n=0; n<prefs.NumTeams; n++) {
    TblSetItemInt(leagueTable, n, 0, TeamResult[n].Position+1);
    TblSetItemInt(leagueTable, n, 1, n);
    TblSetItemInt(leagueTable, n, 2, TeamResult[n].Matches);
    TblSetItemInt(leagueTable, n, 3, TeamResult[n].Goals);
    TblSetItemInt(leagueTable, n, 4, TeamResult[n].AGoals);
    TblSetItemInt(leagueTable, n, 5, TeamResult[n].Points);
    TblSetRowUsable(leagueTable, n, false);
  }
  MemHandleUnlock(TeamResultHandle);

  FrmDrawForm(FrmGetActiveForm());
}

Boolean TableFormHandleEvent(EventPtr e)
{
  Boolean handled;
  
  handled = false;

  switch (e->eType) {

  case frmOpenEvent:
    InitFormTable();
    if (ComputeTable(0xffffffff)) {
      InitTable();
      RedrawTable(leagueTable, TableRememberPosFirst, prefs.NumTeams,
		  ID_FrmTableButton+1, ID_FrmTableButton+2);
    }
    handled  = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlEnter.controlID) {

    // return to results/fixture form
    case ID_FrmTableButton:
      FrmGotoForm(ID_FrmBl);
      handled = true; 
      break;
    
    // "up" button
    case ID_FrmTableButton+1:
      if (TableRememberPosFirst > 0) {
	TableRememberPosFirst -= 10;
	if (TableRememberPosFirst<0) 
	  TableRememberPosFirst = 0;
	RedrawTable(leagueTable,
		    TableRememberPosFirst, 
		    prefs.NumTeams,
		    ID_FrmTableButton+1,
		    ID_FrmTableButton+2);
      }
      handled = true; 
      break;
    
    // "down" button
    case ID_FrmTableButton+2:
      if (TableRememberPosFirst+10 < prefs.NumTeams) {
	TableRememberPosFirst += 10;
	if (TableRememberPosFirst+10 > prefs.NumTeams)
	  TableRememberPosFirst = prefs.NumTeams - 10;
	RedrawTable(leagueTable,
		    TableRememberPosFirst, 
		    prefs.NumTeams,
		    ID_FrmTableButton+1,
		    ID_FrmTableButton+2);
      }
      handled = true; 
      break;
    }
    break;

  case keyDownEvent:
    switch (e->data.keyDown.chr) {
    case pageUpChr:
      if (TableRememberPosFirst > 0) {
	TableRememberPosFirst -= 10;
	if (TableRememberPosFirst<0) 
	  TableRememberPosFirst = 0;
	RedrawTable(leagueTable,
		    TableRememberPosFirst, 
		    prefs.NumTeams,
		    ID_FrmTableButton+1,
		    ID_FrmTableButton+2);
      }
      handled = true; 
      break;

    case pageDownChr:
      if (TableRememberPosFirst+10 < prefs.NumTeams) {
	TableRememberPosFirst += 10;
	if (TableRememberPosFirst+10 > prefs.NumTeams)
	  TableRememberPosFirst = prefs.NumTeams - 10;
	RedrawTable(leagueTable,
		    TableRememberPosFirst, 
		    prefs.NumTeams,
		    ID_FrmTableButton+1,
		    ID_FrmTableButton+2);
      }
      handled = true; 
      break;
    }
    break;

  // tapped on a team name => show statistics
  case tblSelectEvent:
    if ( e->data.tblSelect.column == 1 ) {
      handled = true;
      TeamResult = MemHandleLock(TeamResultHandle);
      stats.team = TeamResult[e->data.tblSelect.row].teamNo;
      MemHandleUnlock(TeamResultHandle);
      FrmPopupForm(ID_FrmStatistics);
    }
    TblUnhighlightSelection(leagueTable);
    break;

  default:
    break;
  }

  return handled;
}

/**
 * UI jump to match day functions
 */

static void InitFormGoto(void)
{
  Int16 row, col;
  gotoTable = Id2Ptr(ID_FrmGoto);
  for (row=0; row<TblGetNumberOfRows(gotoTable); row++) {
    TblSetRowUsable(gotoTable, row, 8*row>=prefs.NumMatchDays?false:true);
    TblSetRowSelectable(gotoTable, row, 
			8*row>=prefs.NumMatchDays?false:true);
    for (col=0; col<8; col++) {
      TblSetColumnUsable(gotoTable, col, true);
      if (row*8+col < prefs.NumMatchDays) {
        TblSetItemStyle(gotoTable, row, col, numericTableItem);
        TblSetItemInt(gotoTable, row, col, row*8+col+1);
      } else
        TblSetItemStyle(gotoTable, row, col, customTableItem);
    }
  }
}

Boolean GotoFormHandleEvent(EventPtr e)
{
  UInt16 r,c;
  Boolean handled;
  
  handled = false;

  switch (e->eType) {

  case frmOpenEvent:
    InitFormGoto();
    FrmDrawForm(FrmGetActiveForm());
    TblDrawTable(gotoTable);
    handled = true; 
    break;

  case tblSelectEvent:
    r = e->data.tblSelect.row;
    c = e->data.tblSelect.column;
    if (r*8+c+1 <= prefs.NumMatchDays) {
      matchDay = r*8 + c + 1;
      FrmGotoForm(ID_FrmBl);
    }
    handled = true;
    break;

  default:
    break;
  }

  return handled;
}

/**
 * About Form
 */
Boolean AboutFormHandleEvent(EventPtr e)
{
  Boolean handled;
  
  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    if (e->data.ctlSelect.controlID == ID_FrmAboutButton) {
      FrmReturnToForm(0);
      handled = true; 
    }
    break;

  default:
    break;
  }

  return handled;
}


/**
 * ShowFixture Form
 */

static void DrawFixtureItem(void* tableP, Int16 row, Int16 column, 
                            RectangleType *bounds)
{
  Int16 n;
  UInt16 x;
  Char buf[25], *p;
  DateTimeType dateP;
  UInt32 date;

  n = TblGetItemInt(tableP, row, column);
  date = TblGetRowData(tableP, row);
  x = bounds->topLeft.x;

  FntSetFont(0);

  switch (column) {

  // date of a match; stored as row data.
  case 0:
    if (date>0) {
      TimSecondsToDateTime(date, &dateP);
      DateToAscii(dateP.month,dateP.day,dateP.year,
		  PrefGetPreference(prefDateFormat), buf);
    } else {
      StrCopy(buf,"");
    }
    break;

  // time of a match; also computed from row data.
  case 1:
    StrCopy(buf,"");
    if (date>0) {
      TimSecondsToDateTime(date, &dateP);
      // no need to show time, if it is "00:00"
      if (dateP.hour+dateP.minute>0) {
	TimeToAscii(dateP.hour,dateP.minute,
		    PrefGetPreference(prefTimeFormat), buf);
      }	  
      x += bounds->extent.x - FntCharsWidth(buf,StrLen(buf)) - 1;
    }
    break;

  // stats.team holds the team for which fixture is shown,
  // item data encodes the match as (home<<8|away)
  // this way home/away is "stored".
  case 2:
    if (n%256 == stats.team) {
      p = GetString(ID_tSTR_away);
      StrCopy(buf, p);
      StrCat(buf, theTeams[n/256-1]);
    } else {
      p = GetString(ID_tSTR_home);
      StrCopy(buf, p);
      StrCat(buf, theTeams[n%256-1]);
    }
    break;

  // items 3 and 4 hold the result as (r1<<8|r2)
  case 3:
    if (n == 0xffff) {
      StrCopy(buf, "");
    } else {
      StrIToA(buf, n/256);
    }
    StrCat(buf," :");
    x += bounds->extent.x - FntCharsWidth(buf,StrLen(buf)) - 1;
    break;
  case 4:
    if (n == 0xffff) {
      StrCopy(buf, "");
    } else {
      StrIToA(buf, n%256);
    }
    break;
  }
  WinSetClip(bounds);
  WinDrawChars(buf, StrLen(buf), x, bounds->topLeft.y);
  WinResetClip();
}  

static Int16 InitFormShowFixture(void)
{
  Int16 col, row, i, j;
  char *p, *q, buf[10];
  matchType *m;
  Int8 numTeams;
  UInt16 res;
  DateTimeType dateP;
  static Int16 widthDate = -1, widthTime;

  fixtureTable = Id2Ptr(ID_FrmShowFixtureTable);

  // from global settings compute maximum needed widths
  // of date and time columns.
  if (widthDate == -1) {
    // the date used here is 2002/12/24 22:30 :-)
    TimSecondsToDateTime(3123613800UL, &dateP);
    DateToAscii(dateP.month,dateP.day,dateP.year,
                PrefGetPreference(prefDateFormat), buf);
    widthDate = FntCharsWidth(buf,StrLen(buf))+1;
    TimeToAscii(dateP.hour,dateP.minute,
		PrefGetPreference(prefTimeFormat), buf);
    widthTime = FntCharsWidth(buf,StrLen(buf))+1;
  }
  TblSetColumnWidth(fixtureTable,0,widthDate);
  TblSetColumnWidth(fixtureTable,1,widthTime);
  TblSetColumnWidth(fixtureTable,2,134-(widthDate+widthTime));

  // set the tables attributes
  for (row=0; row<TblGetNumberOfRows(fixtureTable); row++) {
    TblSetRowSelectable(fixtureTable, row, true);
    TblSetRowUsable(fixtureTable, row, false);
    for (col=0; col<5; col++) {
      TblSetItemStyle(fixtureTable, row, col, customTableItem);
      TblSetColumnUsable(fixtureTable, col, true);
      TblSetCustomDrawProcedure(fixtureTable, col, DrawFixtureItem);
    }
  }

  // and now fill in table data.
  row = 0;
  for (i=0; i<prefs.NumMatchDays; i++) {

    p = q = MemHandleLock(DmQueryRecord(db,i+2));
    numTeams = (Int8)*p;
    p += 2;

    for (j=0; j<numTeams; j++) {
      m = (matchType*)p;
      if ( m->t1 == stats.team || m->t2 == stats.team ) {
	if ( m->t1 == stats.team ) {
	  res = (m->r1<<8)+m->r2;
	} else {
	  res = (m->r2<<8)+m->r1;
	}
	TblSetRowData(fixtureTable, row, m->Date);
	TblSetItemInt(fixtureTable, row, 2, (m->t1<<8)+m->t2);
	TblSetItemInt(fixtureTable, row, 3, res);
	TblSetItemInt(fixtureTable, row, 4, res);
	row++;
      }
      p += sizeof(matchType);
    }
    MemPtrUnlock(q);
  }

  FrmDrawForm(FrmGetActiveForm());
  TblDrawTable(fixtureTable);
  return row;
}

Boolean ShowFixtureFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static Int8 from, rows;
  UInt16 row, column;
  char* title;
  
  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    title = (char*)FrmGetTitle(FrmGetActiveForm());
    while (*title++ != ' ');
    MemMove(title, theTeams[stats.team-1], 
	    StrLen(theTeams[stats.team-1]));
    rows = InitFormShowFixture();
    from = 0;

    RedrawTable(fixtureTable, from,rows,
		ID_FrmShowFixtureButton+1,
		ID_FrmShowFixtureButton+2);

    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {

    // return to statistics form
    case ID_FrmShowFixtureButton:
      FrmGotoForm(ID_FrmStatistics);
      handled = true; 
      break;
    
    // "up" arrow
    case ID_FrmShowFixtureButton+1:
      if (from > 0) {
	from -= 10;
	if (from < 0) from = 0;
      }
      RedrawTable(fixtureTable, from, rows,
		  ID_FrmShowFixtureButton+1,
		  ID_FrmShowFixtureButton+2);
      handled = true; 
      break;
    
    // "down" arrow
    case ID_FrmShowFixtureButton+2:
      if (from+10 < rows) {
	from += 10;
	if (from+10>rows) from = rows - 10;
	RedrawTable(fixtureTable, from, rows,
		    ID_FrmShowFixtureButton+1,
		    ID_FrmShowFixtureButton+2);
      }
      handled = true; 
      break;

    // date book button
    case ID_FrmShowFixtureButton+3: 
      if (DisplayConfirmationMessage(ID_ConfirmationMessages) == 0
	  && OpenDatebook()) {
	Int16 i, j;
	for (i=0; i<prefs.NumMatchDays; i++) {
	  char *p, *q;
	  Int8 numTeams;
	  matchType *m;
	  
	  p = q = MemHandleLock(DmQueryRecord(db,i+2));
	  numTeams = (Int8)*p;
	  p += 2;
	  
	  for (j=0; j<numTeams; j++) {
	    m = (matchType*)p;
	    if ( m->t1 == stats.team || m->t2 == stats.team ) {
	      MakeDatebookEntry(m);
	    }
	    p += sizeof(matchType);
	  }
	  MemPtrUnlock(q);
	}
	DmCloseDatabase(datebook);
      }	

      handled = true; 
      break;

    }
    break;

  case keyDownEvent:
    switch (e->data.keyDown.chr) {
    case pageUpChr:
      if (from>0) {
	from -= 10;
	if (from < 0) from = 0;
	RedrawTable(fixtureTable, from, rows,
		    ID_FrmShowFixtureButton+1,
		    ID_FrmShowFixtureButton+2);
      }
      handled = true; 
      break;

    case pageDownChr:
      if (from+10 < rows) {
	from += 10;
	if (from+10>rows) from = rows - 10;
	RedrawTable(fixtureTable, from, rows,
		    ID_FrmShowFixtureButton+1,
		    ID_FrmShowFixtureButton+2);
      }
      handled = true; 
      break;
    }
    break;

  // show fixture of another team
  case tblSelectEvent:
    TblGetSelection(fixtureTable, &row, &column);
    if (column == 2) {
      UInt16 val = TblGetItemInt(fixtureTable, row, 2);
      if (val%256 == stats.team)
        stats.team = val/256;
      else
        stats.team = val%256;
      FrmGotoForm(ID_FrmShowFixture);
    }
    handled = true;
    break;

  default:
    break;
  }

  return handled;
}


/**
  * Statistics Form Handling Routines
  */
static void DrawIntAt(Int16 v, Coord x, Coord y)
{
  Char buf[35];
  StrIToA(buf,v);
  WinDrawChars(buf,StrLen(buf),x,y);
}

/*
 * build up a string holding the results of matches
 * of a team as characters in terms of won/draw/lost
 * home/away (huh?)
 */
static void GetGamesString(Char* buf, Int16 n)
{
  Int16 i, j;
  Int8 numTeams;
  char *p, *q, *r;
  matchType *m;


  StrCopy(buf,"");
  for (i=stats.first; i<=stats.last; i++) {

    p = q = MemHandleLock(DmQueryRecord(db,i+2));
    numTeams = (Int8)*p;
    p+=2;

    for (j=0; j<numTeams; j++) {
      m = (matchType*)p;
      if (m->t1 == n || m->t2 == n) {

	if (stats.predicate(m,n,i)) {
	  if (m->r1 != 0xff && m->r2 != 0xff) {
	    if (m->t1 == n) { // home match
	      if (m->r1>m->r2) {
		r = GetString(ID_tSTR_H_WON);
	      } else if (m->r1 == m->r2) {
		r = GetString(ID_tSTR_H_DRAW);
	      } else {
		r = GetString(ID_tSTR_H_LOST);
	      }
	    } else { // away match
	      if (m->r2 > m->r1) {
		r = GetString(ID_tSTR_A_WON);
	      } else if (m->r1 == m->r2) {
		r = GetString(ID_tSTR_A_DRAW);
	      } else {
		r = GetString(ID_tSTR_A_LOST);
	      }
	    }
	    StrCat(buf,r);
	  } else {
	    StrCat(buf, "-");
	  }
	}
      }
      p += sizeof(matchType);
    }
    MemPtrUnlock(q);
  }
}

Boolean StatisticsFormHandleEvent(EventPtr e)
{
  Boolean handled;
  Char *title;
  Char buf[91];
  Char Position[3];
  Int16 i, k, team=0;
  
  handled = false;

  switch (e->eType) {

  case frmOpenEvent:
    TeamResult = MemHandleLock(TeamResultHandle);

    for (i=0; i<prefs.NumTeams; i++) {
      if (stats.team == TeamResult[i].teamNo) {
	team = i;
	break;
      }
    }

    FntSetFont(0);
    // modify title, set team name
    title = (char*)FrmGetTitle(FrmGetActiveForm());
    while ( *title++ != ' ' );
    MemMove(title, theTeams[stats.team-1], 
	    StrLen(theTeams[stats.team-1]));
    StrIToA(Position, TeamResult[team].Position+1);
    FldSetTextPtr(Id2Ptr(ID_FrmStatisticsFld), Position);
    FrmDrawForm(FrmGetActiveForm());


    // the rest is done with WinDrawChars because it
    // has rather positive effects on code size; drawback:
    // it's really ugly :-)
    DrawIntAt(TeamResult[team].Matches,5,37);
    DrawIntAt(TeamResult[team].Won,20,37);
    DrawIntAt(TeamResult[team].Rem,35,37);
    DrawIntAt(TeamResult[team].Lost,50,37);
    StrIToA(buf,TeamResult[team].Goals);
    StrCat(buf,":");
    StrIToA(buf+StrLen(buf),TeamResult[team].AGoals);
    WinDrawChars(buf,StrLen(buf),65,37);
    DrawIntAt(TeamResult[team].Goals-TeamResult[team].AGoals,95,37);
    DrawIntAt(TeamResult[team].Points,115,37);
    
    DrawIntAt(TeamResult[team].HMatches,5,71);
    DrawIntAt(TeamResult[team].HWon,20,71);
    DrawIntAt(TeamResult[team].HRem,35,71);
    DrawIntAt(TeamResult[team].HLost,50,71);
    StrIToA(buf,TeamResult[team].HGoals);
    StrCat(buf,":");
    StrIToA(buf+StrLen(buf),TeamResult[team].HAGoals);
    WinDrawChars(buf,StrLen(buf),65,71);
    DrawIntAt(TeamResult[team].HGoals-TeamResult[team].HAGoals,95,71);
    DrawIntAt(TeamResult[team].HPoints,115,71);

    DrawIntAt(TeamResult[team].Matches-TeamResult[team].HMatches,5,105);
    DrawIntAt(TeamResult[team].Won-TeamResult[team].HWon,20,105);
    DrawIntAt(TeamResult[team].Rem-TeamResult[team].HRem,35,105);
    DrawIntAt(TeamResult[team].Lost-TeamResult[team].HLost,50,105);
    StrIToA(buf,TeamResult[team].Goals-TeamResult[team].HGoals);
    StrCat(buf,":");
    StrIToA(buf+StrLen(buf),
	    TeamResult[team].AGoals-TeamResult[team].HAGoals);
    WinDrawChars(buf,StrLen(buf),65,105);
    DrawIntAt(TeamResult[team].Goals-TeamResult[team].AGoals
	      -TeamResult[team].HGoals+TeamResult[team].HAGoals,95,105);
    DrawIntAt(TeamResult[team].Points-TeamResult[team].HPoints,115,105);

    GetGamesString(buf,stats.team);
    k = prefs.NumMatchDays>>1;
    WinDrawChars(buf,(i=StrLen(buf))>k?k:i,5,117);
    if (i>k)
      WinDrawChars(buf+k,StrLen(buf+k),5,128);
    
    handled = true; 
    MemHandleUnlock(TeamResultHandle);
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {

    // return to either table or results form.
    case ID_FrmStatisticsButton:
      FrmReturnToForm(0);
      handled = true; 
      break;

    // show team fixture.
    case ID_FrmStatisticsButton+1:
      FrmGotoForm(ID_FrmShowFixture);
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

  return handled;
}

