#ifndef _BL_H_
#define _BL_H_

/*
 * definitions from main.c
 */
extern UInt32 romversion;
#define FeatureSet20 (romversion >= 0x02003000)
#define FeatureSet30 (romversion >= 0x03003000)
#define FeatureSet31 (romversion >= 0x03103000)
#define FeatureSet35 (romversion >= 0x03503000)

/*
 * We put some functions into different code sections; we do this,
 * because linking with debugging enabled doesn't work anymore. The
 * produced _code_ extends 32KB then. Gdb has slight problems with 
 * debugging code in such a section, but fortunately it exists a 
 * patch, so this is not a problem anymore.
 */

#define EDITFUNC __attribute__ ((__section__("editfns")))
#define DBFUNC   __attribute__ ((__section__("dbfns")))
#define TBLFUNC  __attribute__ ((__section__("tblfns")))

/*
 * definitions from utils.c
 */
void SetField(UInt16 ID, Int16 size, char* val) TBLFUNC;
void* Id2Ptr(UInt16) TBLFUNC;
void FrmMoveObject(Int16, Int16, Int16) TBLFUNC;
void FrmResizeObject(Int16 Id,Int16 width,Int16 height) TBLFUNC;
ListPtr FrmGetListPtr(UInt16 frm, UInt16 id) TBLFUNC;
void ButtonSetLabel(Int16 id, char *l) TBLFUNC;
void DisplayErrorMessage(UInt16 stringId) TBLFUNC;
void DisplayInformationMessage(UInt16 stringId) TBLFUNC;
void DisplayErrorMessageAndStop(UInt16 id) TBLFUNC;
UInt16 DisplayConfirmationMessage(UInt16 id) TBLFUNC;
char* GetString(UInt16 id) TBLFUNC;

/*
 * definitions from bl.c
 */

#define maxNumLeagues 40
#define maxNumTeams 24


typedef struct {
  UInt16 Version;
  Int16 matchDay;
  Int16 NumTeams;
  Int16 NumMatchDays;
  Int8 options;
# define twoPoints  1
# define readonly   2
  Int8 favTeam;
  Int8 pointsWon;
  Int8 pointsDraw;
  Int8 pointsLost;
  Int8 pointsPenalty;
# define tieBreakerNone           0
# define tieBreakerPoints         1
# define tieBreakerGoalsDiff      2
# define tieBreakerGoals          3
# define tieBreakerMatchesWon     4
# define tieBreakerDirectCompare  5
  UInt8 tiebreaker[8];
  struct {
    UInt8 from;
    UInt8 to;
    IndexedColorType color;
  } colors[5];
} BlDatabasePreferencesType;

typedef struct {
  Int16 Version;
  LocalID dbID;
  UInt16 cardNo;
  Int8 options;
# define editMode   1
# define dontSaveEM 2
# define setAlarm   4
  UInt32 dbHash[maxNumLeagues];
  Int16 minutesAlarm;
  IndexedColorType color[5];
} BlApplicationPreferencesType;

extern Int16 DBNo;
extern BlApplicationPreferencesType appPrefs;
extern BlDatabasePreferencesType prefs;

typedef struct {
  Int16 Goals, AGoals, Points,  HGoals, HAGoals, HPoints; 
  Int8 Matches, Position, HMatches, Won, HWon, Rem, HRem, Lost, HLost;
  Int8 teamNo;
} TeamResultType;

typedef struct {
  UInt8 t1, t2, r1, r2;
  UInt32 Date;
} matchTypeOld;

typedef struct {
  UInt8 t1, t2, r1, r2;
  UInt32 Date;
  UInt16 penalty;
} matchType;

typedef struct {
  Int8 type, team;
  Char val;
# define pointHandicap 0
} handicapType;

typedef Boolean (*PredicateF)(matchType*,Int8,Int8);

typedef struct {
  Int16 team;
  Int16 first, last;
  PredicateF predicate;
  Boolean evalHandicap;
} statsType;

extern statsType stats;

typedef Boolean (*matchTypeCallback)(matchType*,Int8);
typedef Boolean (*handicapTypeCallback)(handicapType*,Int8);

extern TeamResultType *TeamResult, *BackupTeamResult;
extern MemHandle TeamResultHandle, BackupTeamResultHandle;

extern Int16 matchDay;

Boolean MatchdayFormHandleEvent(EventPtr) TBLFUNC;
Boolean TableFormHandleEvent(EventPtr) TBLFUNC;
Boolean GotoFormHandleEvent(EventPtr) TBLFUNC;
Boolean AboutFormHandleEvent(EventPtr) TBLFUNC;
Boolean StatisticsFormHandleEvent(EventPtr) TBLFUNC;
Boolean ShowFixtureFormHandleEvent(EventPtr) TBLFUNC;

void DrawMatches(Boolean) TBLFUNC;
Boolean LoopRecords(matchTypeCallback,handicapTypeCallback,UInt32) TBLFUNC;
void RedrawTable(TablePtr, UInt16, Int8, UInt16, UInt16) TBLFUNC;

/*
 * definitions from dbfuncs.c
 */
typedef struct {
  LocalID dbID;
  UInt16 cardNo;
  Char dbName[32];
  UInt16 dbAttr;
} SelectedDBInfo;

extern char* Items[];
extern char* theTeams[];
extern Int16 numOfItems;
extern DmOpenRef db, datebook;

#define tooOld              1
#define noDatabaseInstalled 2

DmOpenRef CreateDatabase(char*) DBFUNC;
Err ReadDatabase(DmOpenRef) DBFUNC;
Err OpenDatabase(UInt16, LocalID) DBFUNC;
void SavePreferences(void) DBFUNC;
void SaveStatus(Int16) DBFUNC;
Int16 FindInstalledDatabases(void) DBFUNC;
void ChooseDB(void) DBFUNC;
void SelectedDB(Int16, SelectedDBInfo*) DBFUNC;
void DeleteRecord(DmOpenRef db, UInt16 index) DBFUNC;
void InsertNewRecord(DmOpenRef, UInt16*, UInt16) DBFUNC;
void InitNewDatabase(DmOpenRef,BlDatabasePreferencesType*) DBFUNC;
Err DeleteDatabase(Int16) DBFUNC;
Boolean RenameDatabase(Int16 i, char* name) DBFUNC;
void swapDbInfo(Int16, Int16) DBFUNC;
void BeamDatabase(Int16) DBFUNC;
Boolean OpenDatebook(void) DBFUNC;
void MakeDatebookEntry(matchType* match) DBFUNC;

/*
 * definitions from edit.c
 */
extern char* TeamsList[];

Boolean EditFormHandleEvent(EventPtr) EDITFUNC;
Boolean TeamsFormHandleEvent(EventPtr) EDITFUNC;
Boolean EnterTeamFormHandleEvent(EventPtr) EDITFUNC;
Boolean NewDBFormHandleEvent(EventPtr) EDITFUNC;
Boolean RenameLeagueFormHandleEvent(EventPtr) EDITFUNC;
Boolean LeagueFormHandleEvent(EventPtr e) EDITFUNC;
void SetTeamsList(void) EDITFUNC;

/*
 * definitions from settings.c
 */
Boolean SettingsFormHandleEvent(EventPtr) EDITFUNC;
Boolean DatabaseSettingsFormHandleEvent(EventPtr) EDITFUNC;
Boolean DatabaseColorsFormHandleEvent(EventPtr) EDITFUNC;

/*
 * definitions from handicaps.c
 */
Boolean HandicapFormHandleEvent(EventPtr) EDITFUNC;
Boolean HandicapEditFormHandleEvent(EventPtr) EDITFUNC;

#endif
