/*
 * Bundesliga is Copyright (c) 1997-2003 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "bl.h"
#include "blrcp.h"

/*
 * helper function to acquire a handle if necessary and set contents
 * of a field.
 */
void SetField(UInt16 ID, Int16 size, char* val)
{
  FieldType* fld;
  MemHandle handle;
  char*  name;

  fld = Id2Ptr(ID);
  handle = FldGetTextHandle(fld);
  FldSetTextHandle(fld, NULL);
  if (!handle) {
    handle = MemHandleNew(size*sizeof(char));
  } else {
    MemHandleResize(handle,size*sizeof(char));
  }
  name = MemHandleLock(handle);
  StrCopy(name, val);
  MemHandleUnlock(handle);
  FldSetTextHandle(fld, handle);
}

/**
 * UI helper function(s)
 */

void* Id2Ptr(UInt16 id)
{ 
  FormPtr frm = FrmGetActiveForm();
  return FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,id));
}

void FrmMoveObject(Int16 Id,Int16 xd,Int16 yd) 
{
  UInt16 x,y;
  FormPtr frm = FrmGetActiveForm();
  UInt16 ObjectIndex = FrmGetObjectIndex(frm ,Id);
  FrmGetObjectPosition(frm, ObjectIndex, &x, &y);
  if (x+xd<160 && y+yd<160)
    FrmSetObjectPosition(frm, ObjectIndex, x+xd, y+yd);
}

void FrmResizeObject(Int16 Id,Int16 width,Int16 height) 
{
  RectangleType bounds;
  FormPtr frm = FrmGetActiveForm();
  UInt16 ObjectIndex = FrmGetObjectIndex(frm ,Id);
  bounds.extent.x = width;
  bounds.extent.y = height;
  FrmGetObjectPosition(frm, ObjectIndex, &bounds.topLeft.x, &bounds.topLeft.y);
  FrmSetObjectBounds(frm, ObjectIndex, &bounds);
}

ListPtr FrmGetListPtr(UInt16 frm, UInt16 id)
{
  FormPtr frmPtr = FrmGetFormPtr(frm);
  return FrmGetObjectPtr(frmPtr, FrmGetObjectIndex(frmPtr, id));
}

void ButtonSetLabel(Int16 id, char *l)
{
  ControlType *ctlP = Id2Ptr(id);
  char *label;
  if (l) {
    label = (char*)CtlGetLabel(ctlP);
    CtlEraseControl(ctlP);
    StrCopy(label,l);
    CtlDrawControl(ctlP);
  }
}

char* GetString(UInt16 id)
{
  char *p;
  MemHandle h = DmGetResource('tSTR', id);
  if (!h) return NULL;
  p = MemHandleLock(h);
  MemHandleUnlock(h);
  return p;
}

void DisplayErrorMessage(UInt16 id)
{
  char *err = GetString(id);
  FrmCustomAlert(ID_ErrorMessagesForm,err,NULL,NULL);
}

void DisplayInformationMessage(UInt16 id)
{
  char *err = GetString(id);
  FrmCustomAlert(ID_InformationMessagesForm,err,NULL,NULL);
}

void DisplayErrorMessageAndStop(UInt16 id)
{
  EventType e;
  DisplayErrorMessage(id);
  e.eType = appStopEvent;
  EvtAddEventToQueue(&e);
}

UInt16 DisplayConfirmationMessage(UInt16 id)
{
  char *msg = GetString(id);
  return FrmCustomAlert(ID_ConfirmationMessagesForm,msg,NULL,NULL);
}

